@@ -28,6 +28,7 @@ from google.protobuf import unittest_features_pb2
 from google.protobuf import unittest_import_pb2
 from google.protobuf import unittest_pb2
 from google.protobuf import unittest_proto3_pb2
+from google.protobuf import unittest_proto3_extensions_pb2
 
 
 TEST_EMPTY_MESSAGE_DESCRIPTOR_ASCII = """
@@ -1351,19 +1352,34 @@ class FeaturesTest(_parameterized.TestCase):
     )
     self.assertEqual(features.json_format, fs.JsonFormat.ALLOW)
 
+  def testProto3ExtensionPresence(self):
+    ext = unittest_proto3_extensions_pb2.Proto3FileExtensions.singular_int
+    file = descriptor_pb2.FileDescriptorProto()
+
+    self.assertFalse(file.options.HasExtension(ext))
+
+    file.options.Extensions[ext] = 1
+
+    self.assertTrue(file.options.HasExtension(ext))
+
+  def testProto3ExtensionHasPresence(self):
+    exts = unittest_proto3_extensions_pb2.Proto3FileExtensions
+    self.assertTrue(exts.singular_int.has_presence)
+    self.assertFalse(exts.repeated_int.has_presence)
+
 
 def GetTestFeature(desc):
   return (
       desc._GetFeatures()
       .Extensions[unittest_features_pb2.test]
-      .int_multiple_feature
+      .multiple_feature
   )
 
 
 def SetTestFeature(proto, value):
   proto.options.features.Extensions[
       unittest_features_pb2.test
-  ].int_multiple_feature = value
+  ].multiple_feature = value
 
 
 @testing_refleaks.TestCase
@@ -1434,21 +1450,29 @@ class FeatureInheritanceTest(unittest.TestCase):
 
     ret = ReturnObject()
     ret.pool = descriptor_pool.DescriptorPool()
+
     defaults = descriptor_pb2.FeatureSetDefaults(
         defaults=[
             descriptor_pb2.FeatureSetDefaults.FeatureSetEditionDefault(
                 edition=descriptor_pb2.Edition.EDITION_PROTO2,
-                features=unittest_pb2.TestAllTypes.DESCRIPTOR._GetFeatures(),
+                overridable_features=unittest_pb2.TestAllTypes.DESCRIPTOR._GetFeatures(),
             )
         ],
         minimum_edition=descriptor_pb2.Edition.EDITION_PROTO2,
         maximum_edition=descriptor_pb2.Edition.EDITION_2023,
     )
-    defaults.defaults[0].features.Extensions[
+    defaults.defaults[0].overridable_features.Extensions[
         unittest_features_pb2.test
-    ].int_multiple_feature = 1
+    ].multiple_feature = 1
     ret.pool.SetFeatureSetDefaults(defaults)
 
+    # Add dependencies
+    file = descriptor_pb2.FileDescriptorProto()
+    descriptor_pb2.DESCRIPTOR.CopyToProto(file)
+    ret.pool.Add(file)
+    unittest_features_pb2.DESCRIPTOR.CopyToProto(file)
+    ret.pool.Add(file)
+
     ret.file = ret.pool.AddSerializedFile(self.file_proto.SerializeToString())
     ret.top_message = ret.pool.FindMessageTypeByName(
         'protobuf_unittest.TopMessage'
