@@ -12,9 +12,9 @@
 #include "google/protobuf/compiler/cpp/file.h"
 
 #include <algorithm>
+#include <cstddef>
 #include <functional>
 #include <memory>
-#include <queue>
 #include <string>
 #include <utility>
 #include <vector>
@@ -27,6 +27,7 @@
 #include "absl/strings/escaping.h"
 #include "absl/strings/match.h"
 #include "absl/strings/str_cat.h"
+#include "absl/strings/str_join.h"
 #include "absl/strings/string_view.h"
 #include "absl/strings/strip.h"
 #include "google/protobuf/compiler/code_generator.h"
@@ -35,12 +36,13 @@
 #include "google/protobuf/compiler/cpp/helpers.h"
 #include "google/protobuf/compiler/cpp/message.h"
 #include "google/protobuf/compiler/cpp/names.h"
+#include "google/protobuf/compiler/cpp/options.h"
 #include "google/protobuf/compiler/cpp/service.h"
 #include "google/protobuf/compiler/retention.h"
-#include "google/protobuf/compiler/scc.h"
 #include "google/protobuf/compiler/versions.h"
 #include "google/protobuf/descriptor.h"
 #include "google/protobuf/descriptor.pb.h"
+#include "google/protobuf/dynamic_message.h"
 #include "google/protobuf/io/printer.h"
 
 // Must be last.
@@ -115,7 +117,7 @@ FileGenerator::FileGenerator(const FileDescriptor* file, const Options& options)
   ABSL_CHECK(msgs_topologically_ordered.size() == msgs.size())
       << "Size mismatch";
 
-  for (int i = 0; i < msgs.size(); ++i) {
+  for (size_t i = 0; i < msgs.size(); ++i) {
     message_generators_.push_back(std::make_unique<MessageGenerator>(
         msgs[i], variables_, i, options, &scc_analyzer_));
     message_generators_.back()->AddGenerators(&enum_generators_,
@@ -164,6 +166,8 @@ void FileGenerator::GenerateFile(io::Printer* p, GeneratedFileType file_type,
   auto guard = IncludeGuard(file_, file_type, options_);
   p->Print(
       "// Generated by the protocol buffer compiler.  DO NOT EDIT!\n"
+      "// NO CHECKED-IN PROTOBUF "
+      "GENCODE\n"
       "// source: $filename$\n");
   if (options_.opensource_runtime) {
     p->Print("// Protobuf C++ Version: $protobuf_cpp_version$\n",
@@ -465,6 +469,8 @@ void FileGenerator::GenerateSourceIncludes(io::Printer* p) {
   absl::StrAppend(&target_basename, options_.proto_h ? ".proto.h" : ".pb.h");
   p->Print(
       "// Generated by the protocol buffer compiler.  DO NOT EDIT!\n"
+      "// NO CHECKED-IN PROTOBUF "
+      "GENCODE\n"
       "// source: $filename$\n");
   if (options_.opensource_runtime) {
     p->Print("// Protobuf C++ Version: $protobuf_cpp_version$\n",
@@ -476,12 +482,15 @@ void FileGenerator::GenerateSourceIncludes(io::Printer* p) {
         #include $h_include$
 
         #include <algorithm>
+        #include <type_traits>
       )");
 
   IncludeFile("third_party/protobuf/io/coded_stream.h", p);
+  IncludeFile("third_party/protobuf/generated_message_tctable_impl.h", p);
   // TODO This is to include parse_context.h, we need a better way
   IncludeFile("third_party/protobuf/extension_set.h", p);
   IncludeFile("third_party/protobuf/wire_format_lite.h", p);
+
   if (ShouldVerify(file_, options_, &scc_analyzer_)) {
     IncludeFile("third_party/protobuf/wire_format_verify.h", p);
   }
@@ -498,10 +507,6 @@ void FileGenerator::GenerateSourceIncludes(io::Printer* p) {
     IncludeFile("third_party/protobuf/wire_format.h", p);
   }
 
-  if (HasGeneratedMethods(file_, options_)) {
-    IncludeFile("third_party/protobuf/generated_message_tctable_impl.h", p);
-  }
-
   if (options_.proto_h) {
     // Use the smaller .proto.h files.
     for (int i = 0; i < file_->dependency_count(); ++i) {
@@ -542,13 +547,8 @@ void FileGenerator::GenerateSourcePrelude(io::Printer* p) {
     PROTOBUF_PRAGMA_INIT_SEG
     namespace _pb = ::$proto_ns$;
     namespace _pbi = ::$proto_ns$::internal;
+    namespace _fl = ::$proto_ns$::internal::field_layout;
   )cc");
-
-  if (HasGeneratedMethods(file_, options_)) {
-    p->Emit(R"cc(
-      namespace _fl = ::$proto_ns$::internal::field_layout;
-    )cc");
-  }
 }
 
 void FileGenerator::GenerateSourceDefaultInstance(int idx, io::Printer* p) {
@@ -838,20 +838,26 @@ void FileGenerator::GenerateSourceForMessage(int idx, io::Printer* p) {
 }
 
 void FileGenerator::GenerateStaticInitializer(io::Printer* p) {
-  if (static_initializers_.empty()) return;
-  p->Emit({{"expr",
-            [&] {
-              for (auto& init : static_initializers_) {
-                init(p);
-              }
-            }}},
-          R"cc(
-            PROTOBUF_ATTRIBUTE_INIT_PRIORITY2
-            static ::std::false_type _static_init_ PROTOBUF_UNUSED =
-                ($expr$, ::std::false_type{});
-          )cc");
-  // Reset the vector because we might be generating many files.
-  static_initializers_.clear();
+  int priority = 0;
+  for (auto& inits : static_initializers_) {
+    ++priority;
+    if (inits.empty()) continue;
+    p->Emit(
+        {{"priority", priority},
+         {"expr",
+          [&] {
+            for (auto& init : inits) {
+              init(p);
+            }
+          }}},
+        R"cc(
+          PROTOBUF_ATTRIBUTE_INIT_PRIORITY$priority$ static ::std::false_type
+              _static_init$priority$_ PROTOBUF_UNUSED =
+                  ($expr$, ::std::false_type{});
+        )cc");
+    // Reset the vector because we might be generating many files.
+    inits.clear();
+  }
 }
 
 void FileGenerator::GenerateSourceForExtension(int idx, io::Printer* p) {
@@ -861,9 +867,13 @@ void FileGenerator::GenerateSourceForExtension(int idx, io::Printer* p) {
 
   NamespaceOpener ns(Namespace(file_, options_), p);
   extension_generators_[idx]->GenerateDefinition(p);
-  static_initializers_.push_back([this, idx](auto* p) {
-    extension_generators_[idx]->GenerateRegistration(p);
-  });
+  for (auto priority : {kInitPriority101, kInitPriority102}) {
+    if (extension_generators_[idx]->WillGenerateRegistration(priority)) {
+      static_initializers_[priority].push_back([this, idx, priority](auto* p) {
+        extension_generators_[idx]->GenerateRegistration(p, priority);
+      });
+    }
+  }
   GenerateStaticInitializer(p);
 }
 
@@ -881,7 +891,7 @@ void FileGenerator::GenerateGlobalSource(io::Printer* p) {
   }
 
   NamespaceOpener ns(Namespace(file_, options_), p);
-  for (int i = 0; i < enum_generators_.size(); ++i) {
+  for (size_t i = 0; i < enum_generators_.size(); ++i) {
     enum_generators_[i]->GenerateMethods(i, p);
   }
 }
@@ -939,13 +949,14 @@ void FileGenerator::GenerateSource(io::Printer* p) {
         )cc");
   }
 
+
   if (IsAnyMessage(file_)) {
     MuteWuninitialized(p);
   }
 
   {
     NamespaceOpener ns(Namespace(file_, options_), p);
-    for (int i = 0; i < message_generators_.size(); ++i) {
+    for (size_t i = 0; i < message_generators_.size(); ++i) {
       GenerateSourceDefaultInstance(
           message_generators_topologically_ordered_[i], p);
     }
@@ -965,12 +976,12 @@ void FileGenerator::GenerateSource(io::Printer* p) {
     // Actually implement the protos
 
     // Generate enums.
-    for (int i = 0; i < enum_generators_.size(); ++i) {
+    for (size_t i = 0; i < enum_generators_.size(); ++i) {
       enum_generators_[i]->GenerateMethods(i, p);
     }
 
     // Generate classes.
-    for (int i = 0; i < message_generators_.size(); ++i) {
+    for (size_t i = 0; i < message_generators_.size(); ++i) {
       p->Emit(R"(
         $hrule_thick$
       )");
@@ -979,7 +990,7 @@ void FileGenerator::GenerateSource(io::Printer* p) {
 
     if (HasGenericServices(file_, options_)) {
       // Generate services.
-      for (int i = 0; i < service_generators_.size(); ++i) {
+      for (size_t i = 0; i < service_generators_.size(); ++i) {
         p->Emit(R"(
           $hrule_thick$
         )");
@@ -989,12 +1000,17 @@ void FileGenerator::GenerateSource(io::Printer* p) {
 
     // Define extensions.
     const auto is_lazily_init = IsLazilyInitializedFile(file_->name());
-    for (int i = 0; i < extension_generators_.size(); ++i) {
+    for (size_t i = 0; i < extension_generators_.size(); ++i) {
       extension_generators_[i]->GenerateDefinition(p);
       if (!is_lazily_init) {
-        static_initializers_.push_back([&, i](auto* p) {
-          extension_generators_[i]->GenerateRegistration(p);
-        });
+        for (auto priority : {kInitPriority101, kInitPriority102}) {
+          if (extension_generators_[i]->WillGenerateRegistration(priority)) {
+            static_initializers_[priority].push_back(
+                [this, i, priority](auto* p) {
+                  extension_generators_[i]->GenerateRegistration(p, priority);
+                });
+          }
+        }
       }
     }
 
@@ -1005,7 +1021,7 @@ void FileGenerator::GenerateSource(io::Printer* p) {
 
   {
     NamespaceOpener proto_ns(ProtobufNamespace(options_), p);
-    for (int i = 0; i < message_generators_.size(); ++i) {
+    for (size_t i = 0; i < message_generators_.size(); ++i) {
       message_generators_[i]->GenerateSourceInProto2Namespace(p);
     }
   }
@@ -1023,9 +1039,74 @@ void FileGenerator::GenerateSource(io::Printer* p) {
   IncludeFile("third_party/protobuf/port_undef.inc", p);
 }
 
+static void GatherAllCustomOptionTypes(
+    const FileDescriptor* file,
+    absl::btree_map<absl::string_view, const Descriptor*>& out) {
+  const DescriptorPool* pool = file->pool();
+  const Descriptor* fd_proto_descriptor = pool->FindMessageTypeByName(
+      FileDescriptorProto::descriptor()->full_name());
+  // Not all pools have descriptor.proto in them. In these cases there for sure
+  // are no custom options.
+  if (fd_proto_descriptor == nullptr) {
+    return;
+  }
+
+  // It's easier to inspect file as a proto, because we can use reflection on
+  // the proto to iterate over all content.
+  // However, we can't use the generated proto linked into the proto compiler
+  // for this, since it doesn't know the extensions that are potentially present
+  // the protos that are being compiled.
+  // Use a dynamic one from the correct pool to parse them.
+  DynamicMessageFactory factory(pool);
+  std::unique_ptr<Message> fd_proto(
+      factory.GetPrototype(fd_proto_descriptor)->New());
+
+  {
+    FileDescriptorProto linkedin_fd_proto;
+    file->CopyTo(&linkedin_fd_proto);
+    ABSL_CHECK(
+        fd_proto->ParseFromString(linkedin_fd_proto.SerializeAsString()));
+  }
+
+  // Now find all the messages used, recursively.
+  std::vector<const Message*> to_process = {fd_proto.get()};
+  while (!to_process.empty()) {
+    const Message& msg = *to_process.back();
+    to_process.pop_back();
+
+    std::vector<const FieldDescriptor*> fields;
+    const Reflection& reflection = *msg.GetReflection();
+    reflection.ListFields(msg, &fields);
+
+    for (auto* field : fields) {
+      if (field->is_extension()) {
+        // Always add the extended.
+        const Descriptor* desc = msg.GetDescriptor();
+        out[desc->full_name()] = desc;
+      }
+
+      // Add and recurse of the extendee if it is a message.
+      const auto* field_msg = field->message_type();
+      if (field_msg == nullptr) continue;
+      if (field->is_extension()) {
+        out[field_msg->full_name()] = field_msg;
+      }
+      if (field->is_repeated()) {
+        for (int i = 0; i < reflection.FieldSize(msg, field); i++) {
+          to_process.push_back(&reflection.GetRepeatedMessage(msg, field, i));
+        }
+      } else {
+        to_process.push_back(&reflection.GetMessage(msg, field));
+      }
+    }
+  }
+}
+
 static std::vector<const Descriptor*>
-GetMessagesToPinGloballyForWeakDescriptors(const FileDescriptor* file) {
-  std::vector<const Descriptor*> out;
+GetMessagesToPinGloballyForWeakDescriptors(const FileDescriptor* file,
+                                           const Options& options) {
+  // Sorted map to dedup and to make deterministic.
+  absl::btree_map<absl::string_view, const Descriptor*> res;
 
   // For simplicity we force pin request/response messages for all
   // services. The current implementation of services might not do
@@ -1035,21 +1116,26 @@ GetMessagesToPinGloballyForWeakDescriptors(const FileDescriptor* file) {
     auto* service = file->service(i);
     for (int j = 0; j < service->method_count(); ++j) {
       auto* method = service->method(j);
-      out.push_back(method->input_type());
-      out.push_back(method->output_type());
+      res[method->input_type()->full_name()] = method->input_type();
+      res[method->output_type()->full_name()] = method->output_type();
     }
   }
 
+  // For correctness, we must ensure that all messages used as custom options in
+  // the descriptor are pinned. Otherwise, we can't properly parse the
+  // descriptor.
+  GatherAllCustomOptionTypes(file, res);
+
+  std::vector<const Descriptor*> out;
+  for (auto& p : res) {
+    // We don't need to pin the bootstrap types. It is wasteful.
+    if (IsBootstrapProto(options, p.second->file())) continue;
+    out.push_back(p.second);
+  }
   return out;
 }
 
 void FileGenerator::GenerateReflectionInitializationCode(io::Printer* p) {
-  if (!message_generators_.empty()) {
-    p->Emit({{"len", message_generators_.size()}}, R"cc(
-      static ::_pb::Metadata $file_level_metadata$[$len$];
-    )cc");
-  }
-
   if (!enum_generators_.empty()) {
     p->Emit({{"len", enum_generators_.size()}}, R"cc(
       static const ::_pb::EnumDescriptor* $file_level_enum_descriptors$[$len$];
@@ -1081,14 +1167,14 @@ void FileGenerator::GenerateReflectionInitializationCode(io::Printer* p) {
         {
             {"offsets",
              [&] {
-               for (int i = 0; i < message_generators_.size(); ++i) {
+               for (size_t i = 0; i < message_generators_.size(); ++i) {
                  offsets.push_back(message_generators_[i]->GenerateOffsets(p));
                }
              }},
             {"schemas",
              [&] {
                int offset = 0;
-               for (int i = 0; i < message_generators_.size(); ++i) {
+               for (size_t i = 0; i < message_generators_.size(); ++i) {
                  message_generators_[i]->GenerateSchema(p, offset,
                                                         offsets[i].second);
                  offset += offsets[i].first;
@@ -1248,13 +1334,10 @@ void FileGenerator::GenerateReflectionInitializationCode(io::Printer* p) {
                            : absl::StrCat(p->LookupVar("desc_table"), "_deps")},
           {"num_deps", num_deps},
           {"num_msgs", message_generators_.size()},
-          {"msgs_ptr", message_generators_.empty()
-                           ? "nullptr"
-                           : std::string(p->LookupVar("file_level_metadata"))},
       },
       R"cc(
         static ::absl::once_flag $desc_table$_once;
-        const ::_pbi::DescriptorTable $desc_table$ = {
+        PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable $desc_table$ = {
             false,
             $eager$,
             $file_proto_len$,
@@ -1267,25 +1350,9 @@ void FileGenerator::GenerateReflectionInitializationCode(io::Printer* p) {
             schemas,
             file_default_instances,
             $tablename$::offsets,
-            $msgs_ptr$,
             $file_level_enum_descriptors$,
             $file_level_service_descriptors$,
         };
-
-        // This function exists to be marked as weak.
-        // It can significantly speed up compilation by breaking up LLVM's SCC
-        // in the .pb.cc translation units. Large translation units see a
-        // reduction of more than 35% of walltime for optimized builds. Without
-        // the weak attribute all the messages in the file, including all the
-        // vtables and everything they use become part of the same SCC through
-        // a cycle like:
-        // GetMetadata -> descriptor table -> default instances ->
-        //   vtables -> GetMetadata
-        // By adding a weak function here we break the connection from the
-        // individual vtables back into the descriptor table.
-        PROTOBUF_ATTRIBUTE_WEAK const ::_pbi::DescriptorTable* $desc_table$_getter() {
-          return &$desc_table$;
-        }
       )cc");
 
   // For descriptor.proto and cpp_features.proto we want to avoid doing any
@@ -1294,19 +1361,18 @@ void FileGenerator::GenerateReflectionInitializationCode(io::Printer* p) {
   // Descriptor initialization will still be performed lazily when it's needed.
   if (!IsLazilyInitializedFile(file_->name())) {
     if (UsingImplicitWeakDescriptor(file_, options_)) {
-      for (auto* pinned : GetMessagesToPinGloballyForWeakDescriptors(file_)) {
-        static_initializers_.push_back([this, pinned](auto* p) {
-          p->Emit(
-              {
-                  {"default", QualifiedDefaultInstanceName(pinned, options_)},
-              },
-              R"cc(
-                ::_pbi::StrongPointer(&$default$),
-              )cc");
+      for (auto* pinned :
+           GetMessagesToPinGloballyForWeakDescriptors(file_, options_)) {
+        static_initializers_[kInitPriority102].push_back([this,
+                                                          pinned](auto* p) {
+          p->Emit({{"pin", StrongReferenceToType(pinned, options_)}},
+                  R"cc(
+                    $pin$,
+                  )cc");
         });
       }
     }
-    static_initializers_.push_back([](auto* p) {
+    static_initializers_[kInitPriority102].push_back([](auto* p) {
       p->Emit(R"cc(
         ::_pbi::AddDescriptors(&$desc_table$),
       )cc");
@@ -1505,23 +1571,30 @@ void FileGenerator::GenerateLibraryIncludes(io::Printer* p) {
     IncludeFile("third_party/protobuf/wire_format_verify.h", p);
   }
 
+  IncludeFile("third_party/protobuf/runtime_version.h", p);
+  int version;
   if (options_.opensource_runtime) {
-    // Verify the protobuf library header version is compatible with the protoc
-    // version before going any further.
-    IncludeFile("third_party/protobuf/port_def.inc", p);
-    p->Emit(
-        {
-            {"version", PROTOBUF_VERSION},
-        },
-        R"(
-      #if PROTOBUF_VERSION != $version$
-      #error "Protobuf C++ gencode is built with an incompatible version of"
-      #error "Protobuf C++ headers/runtime. See"
-      #error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
-      #endif
-    )");
-    IncludeFile("third_party/protobuf/port_undef.inc", p);
+    const auto& v = GetProtobufCPPVersion(/*oss_runtime=*/true);
+    version = v.major() * 1000000 + v.minor() * 1000 + v.patch();
+  } else {
+    version = GetProtobufCPPVersion(/*oss_runtime=*/false).minor();
   }
+  p->Emit(
+      {
+          {"version", version},
+
+          // Downgrade to warnings if version mismatches for bootstrapped files,
+          // so that release_compiler.h can build protoc_minimal successfully
+          // and update stale files.
+          {"err_level", options_.bootstrap ? "warning" : "error"},
+      },
+      R"(
+    #if PROTOBUF_VERSION != $version$
+    #$err_level$ "Protobuf C++ gencode is built with an incompatible version of"
+    #$err_level$ "Protobuf C++ headers/runtime. See"
+    #$err_level$ "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
+    #endif
+  )");
 
   // OK, it's now safe to #include other files.
   IncludeFile("third_party/protobuf/io/coded_stream.h", p);
@@ -1663,7 +1736,7 @@ void FileGenerator::GenerateGlobalStateFunctionDeclarations(io::Printer* p) {
 }
 
 void FileGenerator::GenerateMessageDefinitions(io::Printer* p) {
-  for (int i = 0; i < message_generators_.size(); ++i) {
+  for (size_t i = 0; i < message_generators_.size(); ++i) {
     p->Emit(R"cc(
       $hrule_thin$
     )cc");
@@ -1673,7 +1746,7 @@ void FileGenerator::GenerateMessageDefinitions(io::Printer* p) {
 }
 
 void FileGenerator::GenerateEnumDefinitions(io::Printer* p) {
-  for (int i = 0; i < enum_generators_.size(); ++i) {
+  for (size_t i = 0; i < enum_generators_.size(); ++i) {
     enum_generators_[i]->GenerateDefinition(p);
   }
 }
@@ -1683,7 +1756,7 @@ void FileGenerator::GenerateServiceDefinitions(io::Printer* p) {
     return;
   }
 
-  for (int i = 0; i < service_generators_.size(); ++i) {
+  for (size_t i = 0; i < service_generators_.size(); ++i) {
     p->Emit(R"cc(
       $hrule_thin$
     )cc");
@@ -1716,7 +1789,7 @@ void FileGenerator::GenerateInlineFunctionDefinitions(io::Printer* p) {
       #endif  // __GNUC__
   )");
 
-  for (int i = 0; i < message_generators_.size(); ++i) {
+  for (size_t i = 0; i < message_generators_.size(); ++i) {
     p->Emit(R"cc(
       $hrule_thin$
     )cc");
@@ -1749,7 +1822,7 @@ std::vector<const Descriptor*> FileGenerator::MessagesInTopologicalOrder()
     const {
   std::vector<const Descriptor*> descs;
   descs.reserve(message_generators_.size());
-  for (int i = 0; i < message_generators_.size(); ++i) {
+  for (size_t i = 0; i < message_generators_.size(); ++i) {
     descs.push_back(
         message_generators_[message_generators_topologically_ordered_[i]]
             ->descriptor());
