@@ -10,9 +10,11 @@
 #include <vector>
 
 #include <gtest/gtest.h>
+#include "absl/log/absl_check.h"
 #include "absl/strings/match.h"
 #include "google/protobuf/arena.h"
-#include "google/protobuf/test_util.h"
+#include "google/protobuf/descriptor.h"
+#include "google/protobuf/port.h"
 #include "google/protobuf/text_format.h"
 #include "google/protobuf/unittest.pb.h"
 #include "google/protobuf/unittest_proto3_arena.pb.h"
@@ -100,7 +102,7 @@ TEST(Proto3ArenaTest, Parsing) {
   SetAllFields(&original);
 
   Arena arena;
-  TestAllTypes* arena_message = Arena::CreateMessage<TestAllTypes>(&arena);
+  TestAllTypes* arena_message = Arena::Create<TestAllTypes>(&arena);
   arena_message->ParseFromString(original.SerializeAsString());
   ExpectAllFieldsSet(*arena_message);
 }
@@ -110,7 +112,7 @@ TEST(Proto3ArenaTest, UnknownFields) {
   SetAllFields(&original);
 
   Arena arena;
-  TestAllTypes* arena_message = Arena::CreateMessage<TestAllTypes>(&arena);
+  TestAllTypes* arena_message = Arena::Create<TestAllTypes>(&arena);
   arena_message->ParseFromString(original.SerializeAsString());
   ExpectAllFieldsSet(*arena_message);
 
@@ -130,7 +132,7 @@ TEST(Proto3ArenaTest, GetArena) {
   Arena arena;
 
   // Tests arena-allocated message and submessages.
-  auto* arena_message1 = Arena::CreateMessage<TestAllTypes>(&arena);
+  auto* arena_message1 = Arena::Create<TestAllTypes>(&arena);
   auto* arena_submessage1 = arena_message1->mutable_optional_foreign_message();
   auto* arena_repeated_submessage1 =
       arena_message1->add_repeated_foreign_message();
@@ -139,7 +141,7 @@ TEST(Proto3ArenaTest, GetArena) {
   EXPECT_EQ(&arena, arena_repeated_submessage1->GetArena());
 
   // Tests attached heap-allocated messages.
-  auto* arena_message2 = Arena::CreateMessage<TestAllTypes>(&arena);
+  auto* arena_message2 = Arena::Create<TestAllTypes>(&arena);
   arena_message2->set_allocated_optional_foreign_message(new ForeignMessage());
   arena_message2->mutable_repeated_foreign_message()->AddAllocated(
       new ForeignMessage());
@@ -158,7 +160,7 @@ TEST(Proto3ArenaTest, GetArenaWithUnknown) {
   Arena arena;
 
   // Tests arena-allocated message and submessages.
-  auto* arena_message1 = Arena::CreateMessage<TestAllTypes>(&arena);
+  auto* arena_message1 = Arena::Create<TestAllTypes>(&arena);
   arena_message1->GetReflection()->MutableUnknownFields(arena_message1);
   auto* arena_submessage1 = arena_message1->mutable_optional_foreign_message();
   arena_submessage1->GetReflection()->MutableUnknownFields(arena_submessage1);
@@ -171,7 +173,7 @@ TEST(Proto3ArenaTest, GetArenaWithUnknown) {
   EXPECT_EQ(&arena, arena_repeated_submessage1->GetArena());
 
   // Tests attached heap-allocated messages.
-  auto* arena_message2 = Arena::CreateMessage<TestAllTypes>(&arena);
+  auto* arena_message2 = Arena::Create<TestAllTypes>(&arena);
   arena_message2->set_allocated_optional_foreign_message(new ForeignMessage());
   arena_message2->mutable_repeated_foreign_message()->AddAllocated(
       new ForeignMessage());
@@ -190,8 +192,8 @@ TEST(Proto3ArenaTest, Swap) {
   Arena arena2;
 
   // Test Swap().
-  TestAllTypes* arena1_message = Arena::CreateMessage<TestAllTypes>(&arena1);
-  TestAllTypes* arena2_message = Arena::CreateMessage<TestAllTypes>(&arena2);
+  TestAllTypes* arena1_message = Arena::Create<TestAllTypes>(&arena1);
+  TestAllTypes* arena2_message = Arena::Create<TestAllTypes>(&arena2);
   arena1_message->Swap(arena2_message);
   EXPECT_EQ(&arena1, arena1_message->GetArena());
   EXPECT_EQ(&arena2, arena2_message->GetArena());
@@ -199,7 +201,7 @@ TEST(Proto3ArenaTest, Swap) {
 
 TEST(Proto3ArenaTest, SetAllocatedMessage) {
   Arena arena;
-  TestAllTypes* arena_message = Arena::CreateMessage<TestAllTypes>(&arena);
+  TestAllTypes* arena_message = Arena::Create<TestAllTypes>(&arena);
   TestAllTypes::NestedMessage* nested = new TestAllTypes::NestedMessage;
   nested->set_bb(118);
   arena_message->set_allocated_optional_nested_message(nested);
@@ -208,7 +210,7 @@ TEST(Proto3ArenaTest, SetAllocatedMessage) {
 
 TEST(Proto3ArenaTest, ReleaseMessage) {
   Arena arena;
-  TestAllTypes* arena_message = Arena::CreateMessage<TestAllTypes>(&arena);
+  TestAllTypes* arena_message = Arena::Create<TestAllTypes>(&arena);
   arena_message->mutable_optional_nested_message()->set_bb(118);
   std::unique_ptr<TestAllTypes::NestedMessage> nested(
       arena_message->release_optional_nested_message());
@@ -218,7 +220,7 @@ TEST(Proto3ArenaTest, ReleaseMessage) {
 TEST(Proto3ArenaTest, MessageFieldClear) {
   // GitHub issue #310: https://github.com/protocolbuffers/protobuf/issues/310
   Arena arena;
-  TestAllTypes* arena_message = Arena::CreateMessage<TestAllTypes>(&arena);
+  TestAllTypes* arena_message = Arena::Create<TestAllTypes>(&arena);
   arena_message->mutable_optional_nested_message()->set_bb(118);
   // This should not crash, but prior to the bugfix, it tried to use `operator
   // delete` the nested message (which is on the arena):
@@ -227,7 +229,7 @@ TEST(Proto3ArenaTest, MessageFieldClear) {
 
 TEST(Proto3ArenaTest, MessageFieldClearViaReflection) {
   Arena arena;
-  TestAllTypes* message = Arena::CreateMessage<TestAllTypes>(&arena);
+  TestAllTypes* message = Arena::Create<TestAllTypes>(&arena);
   const Reflection* r = message->GetReflection();
   const Descriptor* d = message->GetDescriptor();
   const FieldDescriptor* msg_field =
@@ -255,6 +257,44 @@ TEST(Proto3OptionalTest, OptionalFields) {
   EXPECT_EQ(serialized.size(), 0);
 }
 
+TEST(Proto3ArenaTest, CheckMessageFieldIsCleared) {
+  Arena arena;
+  auto msg = Arena::Create<TestAllTypes>(&arena);
+
+  // Referring to a saved pointer to a child message is never guaranteed to
+  // work. IOW, protobufs do not guarantee pointer stability. This test only
+  // does this to replicate (unsupported) user behaviors.
+  auto child = msg->mutable_optional_foreign_message();
+  child->set_c(100);
+  msg->Clear();
+
+  EXPECT_EQ(child->c(), 0);
+}
+
+TEST(Proto3ArenaTest, CheckOneofMessageFieldIsCleared) {
+  if (!internal::DebugHardenClearOneofMessageOnArena()) {
+    GTEST_SKIP() << "arena allocated oneof message fields are not hardened.";
+  }
+
+  Arena arena;
+  auto msg = Arena::Create<TestAllTypes>(&arena);
+
+  // Referring to a saved pointer to a child message is never guaranteed to
+  // work. IOW, protobufs do not guarantee pointer stability. This test only
+  // does this to replicate (unsupported) user behaviors.
+  auto child = msg->mutable_oneof_nested_message();
+  child->set_bb(100);
+  msg->Clear();
+
+#ifndef PROTOBUF_ASAN
+  EXPECT_EQ(child->bb(), 0);
+#else
+#if GTEST_HAS_DEATH_TEST && defined(__cpp_if_constexpr)
+  EXPECT_DEATH(EXPECT_EQ(child->bb(), 100), "use-after-poison");
+#endif
+#endif
+}
+
 TEST(Proto3OptionalTest, OptionalFieldDescriptor) {
   const Descriptor* d = protobuf_unittest::TestProto3Optional::descriptor();
 
